/// Copyright (C) 2015  Andrea Cantafio
///
/// This program is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License along
/// with this program; if not, write to the Free Software Foundation, Inc.,
/// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
library cssembed.lib.transformer;

import 'package:barback/barback.dart';
import 'package:path/path.dart';

/// cssembed transformer.
class CssEmbed extends AggregateTransformer {
  CssEmbed.asPlugin();

  @override
  apply(AggregateTransform transform) async {
    Asset cssAsset, htmlAsset;
    await for (Asset asset in transform.primaryInputs) {
      if ('.css' == asset.id.extension) {
        cssAsset = asset;
      }
      if ('.html' == asset.id.extension) {
        htmlAsset = asset;
      }
    }
    if (null == cssAsset || null == htmlAsset) {
      return;
    }
    final css = await cssAsset.readAsString(),
        html = await htmlAsset.readAsString();
    var match = _styleMatcher.firstMatch(html);
    var out;
    if (null == match) {
      match = _moduleMatcher.firstMatch(html);
      if (null == match) {
        return;
      }
      final len = match[0].length;
      final buf = new StringBuffer()
        ..write(html.substring(0, len))
        ..write('\n  <style>\n')
        ..write(css)
        ..write('\n  </style>\n')
        ..write(html.substring(len + 1));
      out = buf.toString();
    } else {
      out = html.replaceFirst(
          _styleMatcher,
          '''
  <style>
    ${match[1]}

    $css
  </style>''');
    }
    transform.consumePrimary(cssAsset.id);
    transform.addOutput(new Asset.fromString(htmlAsset.id, out));
  }

  @override
  classifyPrimary(AssetId id) {
    final path = id.path;
    switch (extension(path)) {
      case '.css':
      case '.html':
        return withoutExtension(path);
      default:
        return null;
    }
  }

  final RegExp _moduleMatcher = new RegExp(r'^<dom-module\s+id="\w+-\w+"\s*>',
          caseSensitive: false, multiLine: true),
      _styleMatcher = new RegExp(r'<style>([^<]*)</style>',
          caseSensitive: false, multiLine: true);
}
